from dataclasses import dataclass
from datetime import datetime
from typing import Any, Dict, Optional

from ..writer import LogWriter
from .base import BaseContainer
from .types import Entity


@dataclass
class RetrievalConfig():
    id: str
    name: Optional[str] = None
    tags: Optional[Dict[str, str]] = None


class Retrieval(BaseContainer):
    def __init__(self, config: RetrievalConfig, writer: LogWriter):
        super().__init__(Entity.RETRIEVAL, config.__dict__, writer)
        self.commit("create")

    def input(self, query: str):
        self.commit("update", {"input": query})
        self.end()

    def output(self, docs: Any):
        self.commit("output", {"docs": docs, "endTimestamp": datetime.now()})
        self.end()



import json
from dataclasses import dataclass, field
from typing import Any, Dict, List, Optional

from ..writer import LogWriter
from .base import BaseContainer
from .types import Entity


@dataclass
class GenerationConfig:
    id: str
    provider: str
    model: str
    messages: List[Any] = field(default_factory=list)
    model_parameters: Dict[str, Any] = field(default_factory=dict)
    span_id: Optional[str] = None
    name: Optional[str] = None
    maxim_prompt_id: Optional[str] = None
    tags: Optional[Dict[str, str]] = None


class Generation(BaseContainer):
    def __init__(self, config: GenerationConfig, writer: LogWriter):
        super().__init__(Entity.GENERATION, config.__dict__, writer)
        self.model = config.model
        self.maxim_prompt_id = config.maxim_prompt_id
        self.messages = []
        self.provider = config.provider
        self.messages.extend(config.messages)
        self.model_parameters = config.model_parameters

    def set_model(self, model: str):
        self.model = model
        self.commit("update", {"model": model})

    def add_message(self, message: Any):
        self.messages.append(message)
        self.commit("update", {"messages": [message]})

    def set_model_parameters(self, model_parameters: Dict[str, Any]):
        self.model_parameters = model_parameters
        self.commit("update", {"model_parameters": model_parameters})

    def result(self, result: Any):
        self.commit("result", {"result": result})
        self.end()

    def data(self) -> Dict[str, Any]:
        base_data = super().data()
        return {
            **base_data,
            "model": self.model,
            "provider": self.provider,
            "maximPromptId": self.maxim_prompt_id,
            "messages": self.messages,
            "modelParameters": self.model_parameters,
        }



# Assuming similar components exist in Python with the same functionality
from ..logger.components.session import Session, SessionConfig
from ..logger.components.trace import Trace, TraceConfig
from .writer import LogWriter, LogWriterConfig


class LoggerConfig:
    def __init__(self, id, auto_flush=False, flush_interval=None):
        self.id = id        
        self.auto_flush = auto_flush
        self.flush_interval = flush_interval

class Logger:
    def __init__(self, config : LoggerConfig, api_key, base_url, is_debug=False):
        if not config.id:
            raise ValueError("Logger must be initialized with id of the logger")
        self._id = config.id
        self.is_debug = is_debug
        writer_config = LogWriterConfig(auto_flush=config.auto_flush,
                                flush_interval=config.flush_interval,
                                base_url=base_url,
                                api_key=api_key,
                                repository_id=config.id)
        self.writer = LogWriter(writer_config)
        self.sessions = {}
        self.traces = {}

    def session(self, config: SessionConfig) -> Session:
        session = self.sessions.get(config.id)
        if session is None:
            session = Session(config, self.writer)
            self.sessions[config.id] = session
        return session

    def trace(self, config: TraceConfig) -> Trace:
        trace = self.traces.get(config.id)
        if trace is None:
            trace = Trace(config, self.writer)
            self.traces[config.id] = trace
        return trace

    @property
    def id(self):
        return self._id

    def cleanup(self):
        # Ending all traces and sessions
        for trace in self.traces.values():
            trace.end()
        for session in self.sessions.values():
            session.end()
        # Cleaning all traces and sessions
        self.sessions = {}
        self.traces = {}
        self.writer.flush()
        self.writer.cleanup()


from dataclasses import dataclass
from typing import Dict, Optional

from ..writer import LogWriter
from .base import EventEmittingBaseContainer
from .feedback import Feedback
from .trace import Trace, TraceConfig
from .types import Entity


@dataclass
class SessionConfig:
    id: str
    name: Optional[str] = None
    tags: Optional[Dict[str, str]] = None


class Session(EventEmittingBaseContainer):
    ENTITY = Entity.SESSION

    def __init__(self, config: SessionConfig, writer: LogWriter):
        super().__init__(Session.ENTITY, config.__dict__, writer)
        self.commit("create")

    def trace(self, config: TraceConfig) -> Trace:
        # Assuming TraceConfig in Python has a session_id attribute
        config.session_id = self.id
        return Trace(config, self.writer)

    def feedback(self, feedback: Feedback):
        self.commit("add-feedback", feedback.__dict__)
